/*! For license information please see dagCode.86544e4d25a0511360e4.js.LICENSE.txt */
!function(e,o){"object"==typeof exports&&"object"==typeof module?module.exports=o():"function"==typeof define&&define.amd?define([],o):"object"==typeof exports?exports.Airflow=o():(e.Airflow=e.Airflow||{},e.Airflow.dagCode=o())}(self,(()=>(()=>{"use strict";var e={r:e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})}},o={};e.r(o);function t(){$(".code pre").toggleClass("wrap")}return"True"===function(e){const o=document.querySelector(`meta[name="${e}"]`);return o&&o.getAttribute("content")||""}("wrapped")&&t(),window.toggleWrap=t,o})()));