print("import: 'airflow'")
import airflow

print("import: 'airflow.api'")
import airflow.api

print("import: 'airflow.api.auth'")
import airflow.api.auth

print("import: 'airflow.api.auth.backend'")
import airflow.api.auth.backend

print("import: 'airflow.api.client'")
import airflow.api.client

print("import: 'airflow.api.common'")
import airflow.api.common

print("import: 'airflow.api.common.experimental'")
import airflow.api.common.experimental

print("import: 'airflow.api_connexion'")
import airflow.api_connexion

print("import: 'airflow.cli'")
import airflow.cli

print("import: 'airflow.cli.commands'")
import airflow.cli.commands

print("import: 'airflow.config_templates'")
import airflow.config_templates

print("import: 'airflow.datasets'")
import airflow.datasets

print("import: 'airflow.dag_processing'")
import airflow.dag_processing

print("import: 'airflow.example_dags'")
import airflow.example_dags

print("import: 'airflow.example_dags.subdags'")
import airflow.example_dags.subdags

print("import: 'airflow.executors'")
import airflow.executors

print("import: 'airflow.hooks'")
import airflow.hooks

print("import: 'airflow.jobs'")
import airflow.jobs

print("import: 'airflow.kubernetes'")
import airflow.kubernetes

print("import: 'airflow.lineage'")
import airflow.lineage

print("import: 'airflow.macros'")
import airflow.macros

print("import: 'airflow.migrations'")
import airflow.migrations

print("import: 'airflow.migrations.versions'")
import airflow.migrations.versions

print("import: 'airflow.models'")
import airflow.models

print("import: 'airflow.mypy'")
import airflow.mypy

print("import: 'airflow.mypy.plugin'")
import airflow.mypy.plugin

print("import: 'airflow.operators'")
import airflow.operators

print("import: 'airflow.providers'")
import airflow.providers

print("import: 'airflow.secrets'")
import airflow.secrets

print("import: 'airflow.security'")
import airflow.security

print("import: 'airflow.sensors'")
import airflow.sensors

print("import: 'airflow.serialization'")
import airflow.serialization

print("import: 'airflow.task'")
import airflow.task

print("import: 'airflow.task.task_runner'")
import airflow.task.task_runner

print("import: 'airflow.ti_deps'")
import airflow.ti_deps

print("import: 'airflow.ti_deps.deps'")
import airflow.ti_deps.deps

print("import: 'airflow.triggers'")
import airflow.triggers

print("import: 'airflow.utils'")
import airflow.utils

print("import: 'airflow.utils.log'")
import airflow.utils.log

print("import: 'airflow.www'")
import airflow.www

print("import: 'airflow.www.api'")
import airflow.www.api

print("import: 'airflow.www.api.experimental'")
import airflow.www.api.experimental

